# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'iosshy/tunneldialog.ui'
#
# Created: Thu Jan 20 08:29:15 2011
#      by: PyQt4 UI code generator 4.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_TunnelDialog(object):
    def setupUi(self, TunnelDialog):
        TunnelDialog.setObjectName(_fromUtf8("TunnelDialog"))
        TunnelDialog.resize(636, 381)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/network-server.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        TunnelDialog.setWindowIcon(icon)
        self.gridLayout = QtGui.QGridLayout(TunnelDialog)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.listTunnels = QtGui.QListWidget(TunnelDialog)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listTunnels.sizePolicy().hasHeightForWidth())
        self.listTunnels.setSizePolicy(sizePolicy)
        self.listTunnels.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.listTunnels.setProperty(_fromUtf8("showDropIndicator"), False)
        self.listTunnels.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        self.listTunnels.setObjectName(_fromUtf8("listTunnels"))
        self.gridLayout.addWidget(self.listTunnels, 0, 0, 1, 1)
        self.verticalLayout = QtGui.QVBoxLayout()
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.grpTunnelProperties = QtGui.QGroupBox(TunnelDialog)
        self.grpTunnelProperties.setEnabled(False)
        self.grpTunnelProperties.setObjectName(_fromUtf8("grpTunnelProperties"))
        self.formLayout = QtGui.QFormLayout(self.grpTunnelProperties)
        self.formLayout.setFieldGrowthPolicy(QtGui.QFormLayout.ExpandingFieldsGrow)
        self.formLayout.setObjectName(_fromUtf8("formLayout"))
        self.lblName = QtGui.QLabel(self.grpTunnelProperties)
        self.lblName.setObjectName(_fromUtf8("lblName"))
        self.formLayout.setWidget(0, QtGui.QFormLayout.LabelRole, self.lblName)
        self.txtName = QtGui.QLineEdit(self.grpTunnelProperties)
        self.txtName.setObjectName(_fromUtf8("txtName"))
        self.formLayout.setWidget(0, QtGui.QFormLayout.FieldRole, self.txtName)
        self.lblPort = QtGui.QLabel(self.grpTunnelProperties)
        self.lblPort.setObjectName(_fromUtf8("lblPort"))
        self.formLayout.setWidget(1, QtGui.QFormLayout.LabelRole, self.lblPort)
        self.txtPort = QtGui.QLineEdit(self.grpTunnelProperties)
        self.txtPort.setObjectName(_fromUtf8("txtPort"))
        self.formLayout.setWidget(1, QtGui.QFormLayout.FieldRole, self.txtPort)
        self.lblLocalPort = QtGui.QLabel(self.grpTunnelProperties)
        self.lblLocalPort.setObjectName(_fromUtf8("lblLocalPort"))
        self.formLayout.setWidget(2, QtGui.QFormLayout.LabelRole, self.lblLocalPort)
        self.txtLocalPort = QtGui.QLineEdit(self.grpTunnelProperties)
        self.txtLocalPort.setObjectName(_fromUtf8("txtLocalPort"))
        self.formLayout.setWidget(2, QtGui.QFormLayout.FieldRole, self.txtLocalPort)
        self.lblCommand = QtGui.QLabel(self.grpTunnelProperties)
        self.lblCommand.setObjectName(_fromUtf8("lblCommand"))
        self.formLayout.setWidget(3, QtGui.QFormLayout.LabelRole, self.lblCommand)
        self.txtCommand = QtGui.QLineEdit(self.grpTunnelProperties)
        self.txtCommand.setObjectName(_fromUtf8("txtCommand"))
        self.formLayout.setWidget(3, QtGui.QFormLayout.FieldRole, self.txtCommand)
        self.chkCloseOnTerm = QtGui.QCheckBox(self.grpTunnelProperties)
        self.chkCloseOnTerm.setObjectName(_fromUtf8("chkCloseOnTerm"))
        self.formLayout.setWidget(4, QtGui.QFormLayout.FieldRole, self.chkCloseOnTerm)
        self.verticalLayout.addWidget(self.grpTunnelProperties)
        self.grpSshProperties = QtGui.QGroupBox(TunnelDialog)
        self.grpSshProperties.setEnabled(False)
        self.grpSshProperties.setObjectName(_fromUtf8("grpSshProperties"))
        self.formLayout_2 = QtGui.QFormLayout(self.grpSshProperties)
        self.formLayout_2.setFieldGrowthPolicy(QtGui.QFormLayout.ExpandingFieldsGrow)
        self.formLayout_2.setObjectName(_fromUtf8("formLayout_2"))
        self.lblHost = QtGui.QLabel(self.grpSshProperties)
        self.lblHost.setObjectName(_fromUtf8("lblHost"))
        self.formLayout_2.setWidget(0, QtGui.QFormLayout.LabelRole, self.lblHost)
        self.txtHost = QtGui.QLineEdit(self.grpSshProperties)
        self.txtHost.setObjectName(_fromUtf8("txtHost"))
        self.formLayout_2.setWidget(0, QtGui.QFormLayout.FieldRole, self.txtHost)
        self.lblSshPort = QtGui.QLabel(self.grpSshProperties)
        self.lblSshPort.setObjectName(_fromUtf8("lblSshPort"))
        self.formLayout_2.setWidget(1, QtGui.QFormLayout.LabelRole, self.lblSshPort)
        self.txtSshPort = QtGui.QLineEdit(self.grpSshProperties)
        self.txtSshPort.setObjectName(_fromUtf8("txtSshPort"))
        self.formLayout_2.setWidget(1, QtGui.QFormLayout.FieldRole, self.txtSshPort)
        self.lblUsername = QtGui.QLabel(self.grpSshProperties)
        self.lblUsername.setObjectName(_fromUtf8("lblUsername"))
        self.formLayout_2.setWidget(2, QtGui.QFormLayout.LabelRole, self.lblUsername)
        self.txtUsername = QtGui.QLineEdit(self.grpSshProperties)
        self.txtUsername.setObjectName(_fromUtf8("txtUsername"))
        self.formLayout_2.setWidget(2, QtGui.QFormLayout.FieldRole, self.txtUsername)
        self.verticalLayout.addWidget(self.grpSshProperties)
        self.gridLayout.addLayout(self.verticalLayout, 0, 1, 1, 1)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.btnAddTunnel = QtGui.QPushButton(TunnelDialog)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Ignored, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnAddTunnel.sizePolicy().hasHeightForWidth())
        self.btnAddTunnel.setSizePolicy(sizePolicy)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/list-add.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnAddTunnel.setIcon(icon1)
        self.btnAddTunnel.setObjectName(_fromUtf8("btnAddTunnel"))
        self.horizontalLayout.addWidget(self.btnAddTunnel)
        self.btnDuplicateTunnel = QtGui.QPushButton(TunnelDialog)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Ignored, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnDuplicateTunnel.sizePolicy().hasHeightForWidth())
        self.btnDuplicateTunnel.setSizePolicy(sizePolicy)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/edit-copy.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnDuplicateTunnel.setIcon(icon2)
        self.btnDuplicateTunnel.setObjectName(_fromUtf8("btnDuplicateTunnel"))
        self.horizontalLayout.addWidget(self.btnDuplicateTunnel)
        self.btnRemoveTunnel = QtGui.QPushButton(TunnelDialog)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Ignored, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnRemoveTunnel.sizePolicy().hasHeightForWidth())
        self.btnRemoveTunnel.setSizePolicy(sizePolicy)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/list-remove.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnRemoveTunnel.setIcon(icon3)
        self.btnRemoveTunnel.setObjectName(_fromUtf8("btnRemoveTunnel"))
        self.horizontalLayout.addWidget(self.btnRemoveTunnel)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 0, 1, 1)
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.btnOK = QtGui.QPushButton(TunnelDialog)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/dialog-ok.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnOK.setIcon(icon4)
        self.btnOK.setDefault(True)
        self.btnOK.setObjectName(_fromUtf8("btnOK"))
        self.horizontalLayout_2.addWidget(self.btnOK)
        self.gridLayout.addLayout(self.horizontalLayout_2, 1, 1, 1, 1)
        self.lblName.setBuddy(self.txtName)
        self.lblPort.setBuddy(self.txtPort)
        self.lblLocalPort.setBuddy(self.txtLocalPort)
        self.lblCommand.setBuddy(self.txtCommand)
        self.lblHost.setBuddy(self.txtHost)
        self.lblSshPort.setBuddy(self.txtHost)
        self.lblUsername.setBuddy(self.txtUsername)

        self.retranslateUi(TunnelDialog)
        QtCore.QObject.connect(self.btnOK, QtCore.SIGNAL(_fromUtf8("clicked()")), TunnelDialog.hide)
        QtCore.QMetaObject.connectSlotsByName(TunnelDialog)
        TunnelDialog.setTabOrder(self.listTunnels, self.btnAddTunnel)
        TunnelDialog.setTabOrder(self.btnAddTunnel, self.btnDuplicateTunnel)
        TunnelDialog.setTabOrder(self.btnDuplicateTunnel, self.btnRemoveTunnel)
        TunnelDialog.setTabOrder(self.btnRemoveTunnel, self.txtName)
        TunnelDialog.setTabOrder(self.txtName, self.txtPort)
        TunnelDialog.setTabOrder(self.txtPort, self.txtLocalPort)
        TunnelDialog.setTabOrder(self.txtLocalPort, self.txtCommand)
        TunnelDialog.setTabOrder(self.txtCommand, self.chkCloseOnTerm)
        TunnelDialog.setTabOrder(self.chkCloseOnTerm, self.txtHost)
        TunnelDialog.setTabOrder(self.txtHost, self.txtSshPort)
        TunnelDialog.setTabOrder(self.txtSshPort, self.txtUsername)
        TunnelDialog.setTabOrder(self.txtUsername, self.btnOK)

    def retranslateUi(self, TunnelDialog):
        TunnelDialog.setWindowTitle(QtGui.QApplication.translate("TunnelDialog", "IOSSHy - Tunnel Configuration", None, QtGui.QApplication.UnicodeUTF8))
        self.grpTunnelProperties.setTitle(QtGui.QApplication.translate("TunnelDialog", "Tunnel Properties", None, QtGui.QApplication.UnicodeUTF8))
        self.lblName.setText(QtGui.QApplication.translate("TunnelDialog", "Name", None, QtGui.QApplication.UnicodeUTF8))
        self.lblPort.setText(QtGui.QApplication.translate("TunnelDialog", "Remote Port", None, QtGui.QApplication.UnicodeUTF8))
        self.txtPort.setInputMask(QtGui.QApplication.translate("TunnelDialog", "00009; ", None, QtGui.QApplication.UnicodeUTF8))
        self.lblLocalPort.setText(QtGui.QApplication.translate("TunnelDialog", "Local Port", None, QtGui.QApplication.UnicodeUTF8))
        self.txtLocalPort.setToolTip(QtGui.QApplication.translate("TunnelDialog", "If local port is set to 0, a random high-numbered port will be selected", None, QtGui.QApplication.UnicodeUTF8))
        self.txtLocalPort.setInputMask(QtGui.QApplication.translate("TunnelDialog", "00009; ", None, QtGui.QApplication.UnicodeUTF8))
        self.lblCommand.setText(QtGui.QApplication.translate("TunnelDialog", "Command", None, QtGui.QApplication.UnicodeUTF8))
        self.txtCommand.setToolTip(QtGui.QApplication.translate("TunnelDialog", "The string {port} in the command line will be replaced by the local port number (useful when the local port is selected by IOSSHy)", None, QtGui.QApplication.UnicodeUTF8))
        self.chkCloseOnTerm.setText(QtGui.QApplication.translate("TunnelDialog", "Automatically close on command termination", None, QtGui.QApplication.UnicodeUTF8))
        self.grpSshProperties.setTitle(QtGui.QApplication.translate("TunnelDialog", "SSH connection options", None, QtGui.QApplication.UnicodeUTF8))
        self.lblHost.setText(QtGui.QApplication.translate("TunnelDialog", "Host", None, QtGui.QApplication.UnicodeUTF8))
        self.lblSshPort.setText(QtGui.QApplication.translate("TunnelDialog", "Port", None, QtGui.QApplication.UnicodeUTF8))
        self.txtSshPort.setInputMask(QtGui.QApplication.translate("TunnelDialog", "00009; ", None, QtGui.QApplication.UnicodeUTF8))
        self.lblUsername.setText(QtGui.QApplication.translate("TunnelDialog", "Username", None, QtGui.QApplication.UnicodeUTF8))
        self.btnAddTunnel.setText(QtGui.QApplication.translate("TunnelDialog", "&Add", None, QtGui.QApplication.UnicodeUTF8))
        self.btnDuplicateTunnel.setText(QtGui.QApplication.translate("TunnelDialog", "&Duplicate", None, QtGui.QApplication.UnicodeUTF8))
        self.btnRemoveTunnel.setText(QtGui.QApplication.translate("TunnelDialog", "&Remove", None, QtGui.QApplication.UnicodeUTF8))
        self.btnOK.setText(QtGui.QApplication.translate("TunnelDialog", "&OK", None, QtGui.QApplication.UnicodeUTF8))

import icons_rc



# -*- coding: utf-8 -*-

import sys, os
from PyQt4.QtGui import QIcon, QDialog, QAction, QKeySequence, QListWidgetItem, QItemSelectionModel
from PyQt4.QtCore import Qt, pyqtSignature, QSettings
from tunnel import Tunnel
from tray import Tray
from Ui_tunneldialog import Ui_TunnelDialog

try:
    from PyKDE4.kdeui import KAboutApplicationDialog, KMainWindow
    import application
    kde = True
    class WindowBaseClass(QDialog, KMainWindow): pass
except ImportError:
    kde = False
    class WindowBaseClass(QDialog): pass

class TunnelDialog(WindowBaseClass, Ui_TunnelDialog):
    def __init__(self):
        WindowBaseClass.__init__(self)
        self.setupUi(self)
        self._explicitQuit = False

        # Tunnels
        self._tunnels = []

        # Setup tray
        self.tray = Tray(self, "IOSSHy", QIcon(":/icons/network-server.png"))
        self.tray.activated.connect(self.activated)

        action = QAction("&Configure", self.tray.menu)
        action.setIcon(QIcon(":/icons/configure.png"))
        action.triggered.connect(self.show)
        self.tray.menu.addAction(action)
        self.tray.menu.setDefaultAction(action)

        self.tray.menu.addSeparator()
        self.actionNoTun = QAction("No tunnels configured", self.tray.menu)
        self.actionNoTun.setEnabled(False)
        self.tray.menu.addAction(self.actionNoTun)
        self.actionLastSep = self.tray.menu.addSeparator()

        if kde:
            action = QAction("&About", self.tray.menu)
            action.triggered.connect(self.about)
            self.tray.menu.addAction(action)

        action = QAction("&Quit", self.tray.menu)
        action.setIcon(QIcon(":/icons/application-exit.png"))
        action.setShortcut(QKeySequence(Qt.CTRL + Qt.Key_Q))
        action.triggered.connect(self.quit)
        self.tray.menu.addAction(action)

        # Load settings
        self.readSettings()
        self.hide()

    def show(self):
        self.visible = True
        WindowBaseClass.show(self)

    def hide(self):
        self.visible = False
        self.writeSettings()
        WindowBaseClass.hide(self)

    def closeEvent(self, event):
        if not self._explicitQuit:
            self.hide()
            event.ignore()

    def on_listTunnels_currentItemChanged(self, current, previous):
        self.grpTunnelProperties.setEnabled(current is not None)
        self.grpSshProperties.setEnabled(current is not None)
        if current is not None:
            tunnel = self._tunnels[self.listTunnels.row(current)]

            self.txtName.setText( tunnel.name )
            self.txtHost.setText( "localhost" if tunnel.host == "" else tunnel.host )
            self.txtLocalPort.setText( "0" if tunnel.localPort is None else str(tunnel.localPort) )
            self.txtPort.setText( str(tunnel.port) )
            self.txtSshPort.setText( str(tunnel.sshPort) )
            self.txtUsername.setText( "root" if tunnel.username == "" else tunnel.username )
            self.txtCommand.setText( "" if tunnel.command is None else tunnel.command )
            self.chkCloseOnTerm.setChecked( Qt.Checked if tunnel.autoClose else Qt.Unchecked )

            self.txtName.setFocus()
            self.txtName.selectAll()
        else:
            self.txtName.setText("")
            self.txtHost.setText("localhost")
            self.txtLocalPort.setText("0")
            self.txtPort.setText("")
            self.txtSshPort.setText("22")
            self.txtUsername.setText("root")
            self.txtCommand.setText("")
            self.chkCloseOnTerm.setChecked(Qt.Unchecked)

    def currentTunnel(self):
        try:
            ti = self.listTunnels.currentRow()
            return None if ti < 0 else self._tunnels[ti]
        except IndexError:
            return None

    def on_txtName_textEdited(self, text):
        self.currentTunnel().name = text

    def on_txtHost_textEdited(self, text):
        self.currentTunnel().host = text

    def on_txtLocalPort_textEdited(self, text):
        self.currentTunnel().localPort = text

    def on_txtPort_textEdited(self, text):
        self.currentTunnel().port = text

    def on_txtSshPort_textEdited(self, text):
        self.currentTunnel().sshPort = text

    def on_txtUsername_textEdited(self, text):
        self.currentTunnel().username = text

    def on_txtCommand_textEdited(self, text):
        self.currentTunnel().command = text

    def on_chkCloseOnTerm_stateChanged(self, state):
        if self.currentTunnel() is not None:
            self.currentTunnel().autoClose = state == Qt.Checked

    @pyqtSignature("")
    def on_btnAddTunnel_clicked(self):
        tunnel = Tunnel(self)
        self._tunnels.append(tunnel)
        self.listTunnels.setCurrentItem(tunnel.item)
        self.tray.menu.insertAction(self.actionLastSep, tunnel.action)
        self.tray.menu.removeAction(self.actionNoTun)

    @pyqtSignature("")
    def on_btnDuplicateTunnel_clicked(self):
        cur = self.currentTunnel()
        if cur is not None:
            tunnel = Tunnel(self)
            tunnel.name = cur.name+" (copy)"
            tunnel.host = cur.host
            tunnel.localPort = cur.localPort
            tunnel.port = cur.port
            tunnel.username = cur.username
            tunnel.command = cur.command
            tunnel.autoClose = cur.autoClose
            self._tunnels.append(tunnel)
            self.listTunnels.setCurrentItem(tunnel.item)
            self.tray.menu.insertAction(self.actionLastSep, tunnel.action)

    @pyqtSignature("")
    def on_btnRemoveTunnel_clicked(self):
        tunnel = self.currentTunnel()
        if tunnel is not None:
            ti = self.listTunnels.currentRow()
            del self._tunnels[ti]
            self.listTunnels.setCurrentRow(0 if ti != 0 else 1)
            self.listTunnels.takeItem(ti)
            self.listTunnels.setCurrentItem(None, QItemSelectionModel.Clear)
            self.tray.menu.removeAction(tunnel.action)
            del tunnel

            if len(self._tunnels) == 0:
                self.tray.menu.insertAction(self.actionLastSep, self.actionNoTun)

    def activated(self):
        if self.visible:
            self.hide()
        else:
            self.show()

    def readSettings(self):
        if os.name == 'nt':
            settings = QSettings()
        else:
            settings = QSettings(os.path.expanduser("~/.iosshy.ini"), QSettings.IniFormat)
        for name in settings.childGroups():
            tunnel = Tunnel(self)
            tunnel.name = name
            tunnel.readSettings(settings)
            self._tunnels.append(tunnel)
            self.tray.menu.insertAction(self.actionLastSep, tunnel.action)
            self.tray.menu.removeAction(self.actionNoTun)

    def writeSettings(self):
        if os.name == 'nt':
            settings = QSettings()
        else:
            settings = QSettings(os.path.expanduser("~/.iosshy.ini"), QSettings.IniFormat)
        settings.clear()
        for tunnel in self._tunnels:
            tunnel.writeSettings(settings)

    def about(self):
        KAboutApplicationDialog(application.aboutData, self).exec_()
        if self.isHidden():
            self.show()
            self.hide()

    def quit(self):
        self.writeSettings()
        self._explicitQuit = True
        self.close()



# -*- coding: utf-8 -*-

from PyQt4.QtCore import QObject, pyqtSignal
from PyQt4.QtGui import QIcon
try:
    from PyKDE4.kdeui import KStatusNotifierItem, KMenu
    kde = True
except ImportError:
    from PyQt4.QtGui import QSystemTrayIcon, QMenu
    kde = False

class Tray(QObject):
    activated = pyqtSignal()

    def __init__(self, parent, title, icon):
        QObject.__init__(self)

        # Setup contextual menu
        if kde:
            self.menu = KMenu(parent)
            self.tray = KStatusNotifierItem(parent)
            self.tray.setStatus(KStatusNotifierItem.Active)
            self.tray.setCategory(KStatusNotifierItem.ApplicationStatus)
            self.tray.setAssociatedWidget(parent)
            self.tray.setStandardActionsEnabled(False)
            self.tray.activateRequested.connect(self._activateRequested)
        else:
            self.menu = QMenu()
            self.tray = QSystemTrayIcon()
            self.tray.activated.connect(self._activated)
        self.setIcon(icon)
        self.setTitle(title)
        if not kde:
            self.tray.show()
        self.tray.setContextMenu(self.menu)

    def setTitle(self, title):
        if kde:
            self.tray.setTitle(title)
            self.tray.setToolTipTitle(title)
        else:
            self.tray.setToolTip(title)
        self.menu.setTitle(title)

    def setIcon(self, icon):
        if kde:
            self.tray.setIconByPixmap(icon)
            self.tray.setToolTipIconByPixmap(icon)
        else:
            self.tray.setIcon(icon)

    def showMessage(self, title, message, icon=None):
        if kde:
            self.tray.showMessage(title, message, "network-server")
        else:
            self.tray.showMessage(title, message, QSystemTrayIcon.Information if icon is None else icon)

    def _activated(self, reason):
        if reason == QSystemTrayIcon.DoubleClick:
            self.activated.emit()

    def _activateRequested(self, active, pos):
        self.activated.emit()




# -*- coding: utf-8 -*-

import sip, os, sys
sip.setapi("QString", 2)
sip.setapi("QVariant", 2)

import warnings
warnings.filterwarnings("ignore", ".*sha module is deprecated.*", DeprecationWarning)
warnings.filterwarnings("ignore", ".*md5 module is deprecated.*", DeprecationWarning)
warnings.filterwarnings("ignore", ".*This application uses RandomPool.*", DeprecationWarning)

app = None
aboutData = None

name = "IOSSHy"
description = "Desktop tool to quickly setup SSH tunnels and automatically execute commands that make use of them"
version = "1.5"
url = "http://github.com/mtorromeo/iosshy"

def main():
    global app, aboutData

    import setproctitle
    setproctitle.setproctitle("iosshy")

    from PyQt4.QtCore import QCoreApplication, QTranslator, QLocale, QSettings
    from PyQt4.QtGui import QApplication, QSystemTrayIcon, QImage

    from tunneldialog import TunnelDialog

    try:
        from PyKDE4.kdecore import ki18n, KAboutData, KCmdLineArgs
        from PyKDE4.kdeui import KApplication, KIcon

        aboutData = KAboutData(
            name, #appName
            name, #catalogName
            ki18n(name), #programName
            version,
            ki18n(description), #shortDescription
            KAboutData.License_BSD, #licenseKey
            ki18n("© 2010 Massimiliano Torromeo"), #copyrightStatement
            ki18n(""), #text
            url #homePageAddress
        )
        aboutData.setBugAddress("http://github.com/mtorromeo/iosshy/issues")
        aboutData.addAuthor(
            ki18n("Massimiliano Torromeo"), #name
            ki18n("Main developer"), #task
            "massimiliano.torromeo@gmail.com" #email
        )
        aboutData.setProgramLogo(QImage(":icons/network-server.png"))

        KCmdLineArgs.init(sys.argv, aboutData)

        app = KApplication()
        app.setWindowIcon(KIcon("network-server"))

        if app.isSessionRestored():
            sys.exit(0)
    except ImportError:
        app = QApplication(sys.argv)
        app.setOrganizationName("MTSoft")
        app.setApplicationName(name)


    if QSystemTrayIcon.isSystemTrayAvailable():
        translator = QTranslator()
        qmFile = "tunneller_%s.qm" % QLocale.system().name()
        if os.path.isfile(qmFile):
            translator.load(qmFile)
        app.installTranslator(translator)

        dialog = TunnelDialog()
        sys.exit(app.exec_())
    else:
        print "System tray not available. Exiting."
        sys.exit(1)

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
py2app/py2exe build script for IOSSHy

Will automatically ensure that all build prerequisites are available
via ez_setup

Usage (Mac OS X):
    python setup.py py2app

Usage (Windows):
    python setup.py py2exe
"""

# Include iosshy application first to force sip api version 2
from iosshy import application
import sys, os
from setuptools import setup
#from distutils.core import setup

mainscript = 'bin/iosshy'
README = os.path.join(os.path.dirname(__file__), 'README.rst')

# Programmatically compile the forms with the uic module, sadly this is not possible for icon resources
from PyQt4 import uic
for form in ("tunneldialog.ui",):
	uif = "Ui_{0}.py".format( form.rpartition('.')[0] )
	with open(os.path.join("iosshy", uif), "wb") as f:
		uic.compileUi(os.path.join("iosshy", form), f)

if sys.platform == 'darwin':
	extra_options = dict(
		setup_requires=['py2app'],
		app=[mainscript],
		options={"py2app": {
			"argv_emulation": True,
			"includes": ["sip", "PyQt4.QtCore", "PyQt4.QtGui"]
		}},
	)
elif sys.platform == 'win32':
	import py2exe

	# Override isSystemDLL function to force inclusion of msvcp90.dll
	origIsSystemDLL = py2exe.build_exe.isSystemDLL
	def isSystemDLL(pathname):
		if os.path.basename(pathname).lower() == "msvcp90.dll":
			return 0
		return origIsSystemDLL(pathname)
	py2exe.build_exe.isSystemDLL = isSystemDLL

	extra_options = dict(
		windows=[{
			"script": mainscript,
			"icon_resources": [(0, "iosshy.ico")]
		}],
		options={"py2exe": {
			"optimize": 2,
			"skip_archive": True,
			"includes": ["sip"]
		}}
	)
else:
	extra_options = {}

setup(
	name = application.name,
	packages = ["iosshy"],
	scripts = [mainscript],
	requires = ["paramiko", "keyring", "setproctitle"],
	version = application.version,
	description = application.description,
	long_description = open(README).read(),
	author = "Massimiliano Torromeo",
	author_email = "massimiliano.torromeo@gmail.com",
	url = application.url,
	download_url = "http://github.com/mtorromeo/iosshy/tarball/v"+application.version,
	keywords = ["qt", "pyqt", "desktop", "ssh"],
	classifiers = [
		"Programming Language :: Python",
		"Programming Language :: Python :: 2.6",
		"Programming Language :: Python :: 2.7",
		"Development Status :: 5 - Production/Stable",
		"License :: OSI Approved :: BSD License",
		"Environment :: X11 Applications :: Qt",
		"Intended Audience :: System Administrators",
		"Operating System :: MacOS :: MacOS X",
		"Operating System :: Microsoft :: Windows",
		"Operating System :: POSIX :: Linux",
		"Operating System :: POSIX :: BSD",
		"Natural Language :: English",
		"Topic :: System :: Monitoring",
		"Topic :: Utilities"
	],
	license = "BSD License",
	**extra_options
)



# -*- coding: utf-8 -*-

import paramiko, select, SocketServer
from threading import Thread
from subprocess import Popen
import keyring
from PyQt4.QtCore import Qt, QObject, pyqtSignal
from PyQt4.QtGui import QInputDialog, QLineEdit
try:
	import Crypto.Random as Random
except ImportError as e:
	Random = None

class ForwardServer(SocketServer.ThreadingTCPServer):
	daemon_threads = True
	allow_reuse_address = True

class Handler(SocketServer.BaseRequestHandler):
	def handle(self):
		try:
			chan = self.ssh_transport.open_channel('direct-tcpip', (self.chain_host, self.chain_port), self.request.getpeername())
		except Exception, e:
			return

		if chan is None:
			return

		#verbose('Connected! Tunnel open %r -> %r -> %r' % (self.request.getpeername(), chan.getpeername(), (self.chain_host, self.chain_port)))
		while True:
			r, w, x = select.select([self.request, chan], [], [])
			if self.request in r:
				data = self.request.recv(1024)
				if len(data) == 0:
					break
				chan.send(data)
			if chan in r:
				data = chan.recv(1024)
				if len(data) == 0:
					break
				self.request.send(data)
		chan.close()
		self.request.close()

class TunnelThread(Thread):
	def __init__(self, ssh_server, local_port=0, ssh_port=22, remote_host="localhost", remote_port=None, username=None, password=None):
		Thread.__init__(self)
		if Random is not None:
			Random.atfork()
		if remote_port is None:
			remote_port = local_port
		self.local_port = local_port
		self.remote_host = remote_host
		self.remote_port = remote_port

		self.ssh_client = paramiko.SSHClient()
		self.ssh_client.load_system_host_keys()
		self.ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
		self.ssh_client.connect(ssh_server, ssh_port, username=username, password=password, look_for_keys=True)

		transport = self.ssh_client.get_transport()

		class SubHandler(Handler):
			chain_host = remote_host
			chain_port = remote_port
			ssh_transport = transport
		self.ffwd_server = ForwardServer(('', self.local_port), SubHandler)
		self.ip, self.local_port = self.ffwd_server.server_address

	def run(self):
		self.ffwd_server.serve_forever()

	def join(self):
		if self.ffwd_server is not None:
			self.ffwd_server.shutdown()
		self.ssh_client.close()
		del self.ffwd_server
		del self.ssh_client
		Thread.join(self)

class CommandThread(Thread, QObject):
	terminated = pyqtSignal()

	def __init__(self, command):
		Thread.__init__(self)
		QObject.__init__(self)
		self.pipe = Popen(command, shell=True)

	def run(self):
		self.pipe.communicate()
		self.terminated.emit()

from PyQt4.QtGui import QAction, QListWidgetItem, QSystemTrayIcon
from PyQt4.QtCore import Qt

class Tunnel(object):
	def __init__(self, parent):
		self._thread = None
		self._commandThread = None
		self._parent = parent
		self._name = "New Tunnel"
		self.host = "localhost"
		self._localPort = 0
		self._port = 80
		self._sshPort = 22
		self.username = "root"
		self.command = None
		self.autoClose = False
		self._action = QAction(self._name, self._parent.tray.menu)
		self._action.setCheckable(True)
		self._action.toggled.connect(self.toggle)
		self._item = QListWidgetItem(self._name, self._parent.listTunnels)

	def _validatePort(self, port):
		try:
			if type(port) is str:
				port = port.strip()
			port = int(port)
		except:
			return None
		else:
			return port if 0 < port < 65536 else None

	def getAction(self):
		return self._action

	def getItem(self):
		return self._item

	def getName(self):
		return self._name

	def setName(self, name):
		self._name = name
		self._action.setText(name)
		self._item.setText(name)

	def getPort(self):
		return self._port

	def setPort(self, port):
		port = self._validatePort(port)
		if port is not None:
			self._port = port

	def getLocalPort(self):
		return self._localPort

	def setLocalPort(self, port):
		self._localPort = self._validatePort(port)
		if self._localPort is None:
			self._localPort = 0

	def getSshPort(self):
		return self._sshPort

	def setSshPort(self, port):
		port = self._validatePort(port)
		if port is not None:
			self._sshPort = port

	name = property(getName, setName)
	port = property(getPort, setPort)
	localPort = property(getLocalPort, setLocalPort)
	sshPort = property(getSshPort, setSshPort)
	action = property(getAction)
	item = property(getItem)

	def readSettings(self, settings):
		settings.beginGroup(self.name)
		self.host = settings.value("host", "localhost")
		self.sshPort = settings.value("sshPort", "22")
		self.localPort = settings.value("localPort", None)
		self.port = settings.value("port", None)
		self.username = settings.value("username", "root")
		self.command = settings.value("command", None)
		self.autoClose = settings.value("autoClose", False)
		settings.endGroup()

	def writeSettings(self, settings):
		settings.beginGroup(self.name)
		settings.setValue("host", self.host)
		settings.setValue("sshPort", self.sshPort)
		if self.localPort is not None:
			settings.setValue("localPort", self.localPort)
		settings.setValue("port", self.port)
		settings.setValue("username", self.username)
		if self.command is not None:
			settings.setValue("command", self.command)
		settings.setValue("autoClose", self.autoClose)
		settings.endGroup()

	def toggle(self, openTunnel):
		if openTunnel:
			openTunnel = self._thread is None
		if openTunnel:
			self.open_()
		else:
			self.close()

	def open_(self):
		connectionName = "{0}@{1}:{2}".format(self.username, self.host, self.sshPort)
		password = keyring.get_password("ssh", connectionName)
		try2connect = True
		while try2connect:
			try2connect = False
			try:
				self._thread = TunnelThread(username=self.username, password=password, ssh_server=self.host, ssh_port=self.sshPort, local_port=self.localPort, remote_port=self.port)
			except paramiko.BadHostKeyException as e:
				self.close(str(e), QSystemTrayIcon.Warning)
			except (paramiko.PasswordRequiredException, paramiko.AuthenticationException) as e:
				(password, ok) = QInputDialog.getText(self._parent, "Password required for {0}".format(connectionName), "Insert password for {0}:".format(connectionName), QLineEdit.Password, "", Qt.Tool)
				if ok:
					keyring.set_password("ssh", connectionName, password)
					try2connect = True
				else:
					self.close()
			except Exception as e:
				self.close(type(e).__name__+": "+str(e), QSystemTrayIcon.Critical)
			else:
				self._parent.tray.showMessage(self.name, "Tunnel active")

		if self._thread is not None:
			self._thread.start()
			if self.command is not None:
				command = self.command
				try:
					command = command.format(port=self._thread.local_port)
				except: pass
				try:
					self._commandThread = CommandThread(command)
					if self.autoClose:
						self._commandThread.terminated.connect(self.close)
					self._commandThread.start()
				except:
					self._commandThread = None
					if self.autoClose:
						self.close()

	def close(self, message="Closing tunnel", icon=QSystemTrayIcon.Information):
		if self._thread is not None:
			self._parent.tray.showMessage(self.name, message, icon)
			if self._commandThread is not None:
				self._commandThread.join()
				self._commandThread = None
			self._thread.join()
			self._thread = None
		self._action.setChecked(False)